#pragma once

#include <QObject>
#include <QList>
#include <QMap>
#include <QTimer>

#include "qsacn_global.h"
#include "universe.h"

/**
 * @brief The QSacnUniverse class
 *
 * Rather than QSacnNode reimplemnting the sACN::Node api for a QSacnUniverse that uses
 * sACN::Universe as a base class, the QSacnUniverse class wraps a sACN::Universe.
 */
class QT_EXPORT QSacnUniverse
        : public QObject
{
    Q_OBJECT
public:
  explicit QSacnUniverse(QObject *parent = nullptr,
                         std::shared_ptr<sACN::Universe> universe = nullptr);
  virtual ~QSacnUniverse();

  const QString sourceName() const;
  uint16_t number() const;
  uint8_t  priority() const;
  double   rxRate() const;
  uint8_t  status() const;
  uint8_t  slot(const uint16_t) const;
  uint16_t activeSlots() const;
  bool isEditable() const;
  const QList<sACN::DATA::data_header> sources() const;
  QSacnUniverse* sourceUniverse(const sACN::DATA::data_header&) const;

public slots:
  void setOptions(sACN::DATA::data_options o);
  void setPriority(uint8_t p);
  void setSyncAddress(uint16_t a);
  void setValue (const uint16_t addr, const uint8_t level);
  void setValue (const uint16_t addr, const uint16_t size,
                 const uint8_t* profile);
  void syncSources();

signals:
  void dataChanged();
  void sourceListChanged();
  void statusChanged();

private:
  std::shared_ptr<sACN::Universe> universe_;
  QMap<sACN::DATA::data_header, QSacnUniverse*> sources_;

  std::shared_ptr<void> data_change_token;
  std::shared_ptr<void> list_change_token;
  std::shared_ptr<void> status_change_token;

  QTimer *status_watchdog_;
};
Q_DECLARE_METATYPE(QSacnUniverse*)
